//------------------------------------------------------------------------------
#include <SPI.h>
#include <Ethernet.h>
#include <PubSubClient.h>

const char *TOPIC_STATE = "state/relay";
const char *TOPIC_CONTROL = "control/relay";

// Set relays pins variables to Arduino digital pins
const int relay1 = 2;
const int relay2 = 3;
const int relay3 = 5; // pin 4 used by ethernet shield
const int relay4 = 6;
const int relay5 = 7;
const int relay6 = 8;
const int relay7 = 9;
const int relay8 = 10; // pins 11,12,13 used by ethernetshield

// Relays state
bool relay1State = LOW;
bool relay2State = LOW;
bool relay3State = LOW;
bool relay4State = LOW;
bool relay5State = LOW;
bool relay6State = LOW;
bool relay7State = LOW;
bool relay8State = LOW;

//---------------------------------------------------------------------------

// Arduino MAC address is on a sticker on your Ethernet shield
// must be unique for every node in same network
// To make a new unique address change last letter

byte mac[] = {0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xEE};

// Unique static IP address of this Arduino - change to adapt to your network
IPAddress ip(192, 168, 0, 238);

// IP Address of your MQTT broker - change to adapt to your network
byte server[] = {192, 168, 0, 20};

// Handle and convert incoming MQTT messages ----------------------------------------
void initRelaysPins();
void callback(char *topic, byte *payload, unsigned int length);
void turnOnByCommand(String content);
void turnOffByCommand(String content);
void turnOnRelay(int relayPin, char *content);
void turnOffRelay(int relayPin, char *content);
// Initiate instances -----------------------------------

EthernetClient ethClient;
PubSubClient client(server, 1883, callback, ethClient);

//-------------------------------------------------------

void setup()
{
  initRelaysPins();

  // Setup serial connection
  Serial.begin(9600);

  // Setup ethernet connection to MQTT broker
  Ethernet.begin(mac);
  if (client.connect("arduino - ip - 238"))
  { // change as desired - clientname must be unique for MQTT broker
    client.publish(TOPIC_STATE, "hello world - here arduino ip 239");
    Serial.println("connected");
    client.subscribe(TOPIC_CONTROL); // subscribe to topic “led”
  }
}

//----------------------------------------------

void loop()
{
  client.loop();
}

void callback(char *topic, byte *payload, unsigned int length)
{
  // handle message arrived
  String content = "";
  char character;
  for (int num = 0; num < length; num++)
  {
    character = payload[num];
    content.concat(character);
  }
  Serial.println(topic);
  Serial.println(content); // message sent out by button actions is returned from broker and serial printed

  // Set specific virtual switches on basis of specific incoming messages ----------------------------

  turnOnByCommand(content);
  turnOffByCommand(content);
}

void initRelaySPins()
{
  pinMode(relay1, OUTPUT);
  pinMode(relay2, OUTPUT);
  pinMode(relay3, OUTPUT);
  pinMode(relay4, OUTPUT);
  pinMode(relay5, OUTPUT);
  pinMode(relay6, OUTPUT);
  pinMode(relay7, OUTPUT);
  pinMode(relay8, OUTPUT);
}

void turnOnByCommand(String content)
{
  if (content == "1on")
  {
    //tutaj mozna pomysleć nad zamiana content na typ char *
    turnOnRelay(relay1, "1on");
  }
  else if (content == "2on")
  {
    turnOnRelay(relay1, "2on");
  }
  else if (content == "3on")
  {
    turnOnRelay(relay1, "3on");
  }
  else if (content == "4on")
  {
    turnOnRelay(relay1, "4on");
  }
  else if (content == "5on")
  {
    turnOnRelay(relay1, "5on");
  }
  else if (content == "6on")
  {
    turnOnRelay(relay1, "6on");
  }
  else if (content == "7on")
  {
    turnOnRelay(relay1, "7on");
  }
  else if (content == "8on")
  {
    turnOnRelay(relay1, "8on");
  }
}

void turnOffByCommand(String content)
{
  if (content == "1off")
  {
    //tutaj mozna pomysleć nad zamiana content na typ char *
    turnOffRelay(relay1, "1off");
  }
  else if (content == "2off")
  {
    turnOffRelay(relay1, "2off");
  }
  else if (content == "3off")
  {
    turnOffRelay(relay1, "3off");
  }
  else if (content == "4off")
  {
    turnOffRelay(relay1, "4off");
  }
  else if (content == "5off")
  {
    turnOffRelay(relay1, "5off");
  }
  else if (content == "6off")
  {
    turnOffRelay(relay1, "6off");
  }
  else if (content == "7off")
  {
    turnOffRelay(relay1, "7off");
  }
  else if (content == "8off")
  {
    turnOffRelay(relay1, "8off");
  }
}

void turnOnRelay(int relayPin, char *content)
{
  if (client.connected())
  {
    digitalWrite(relayPin, HIGH);
    client.publish(TOPIC_STATE, content, strlen(content));
  }
}

void turnOffRelay(int relayPin, char *content)
{
  if (client.connected())
  {
    digitalWrite(relayPin, LOW);
    client.publish(TOPIC_STATE, content, strlen(content));
  }
}
// End of sketch ---------------------------------